@@ -2,7 +2,7 @@
 /**
  * This file is part of BibORB
  *
- * Copyright (C) 2003-2004  Guillaume Gardey (ggardey@club-internet.fr)
+ * Copyright (C) 2003-2005  Guillaume Gardey (ggardey@club-internet.fr)
  * 
  * BibORB is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,16 +20,18 @@
  * 
  */
 
-/** 
-    File: auth.php
-    Author: Guillaume Gardey (ggardey@club-internet.fr)
-    Licence: GPL
-
+/**
+ * File: auth.php
+ * Author: Guillaume Gardey (ggardey@club-internet.fr)
+ * Licence: GPL
+ *
+ *    Redirect to the php source matching the authentication method
+ *    defined in the configuration file of BibORB.
  */
-require_once("config.php");
 
-if(!$GLOBALS['disable_authentication']){
-    switch($GLOBALS['authentication_method']){
+require_once("config.php");
+if(!DISABLE_AUTHENTICATION){
+    switch(AUTH_METHOD){
         // Use file authentication system
         case 'files':
             require_once("php/auth_backends/auth.file.php");
